-module(ssh_delay_test).

%% API
-export([test_setopts/0, test_getopts/0]).

% path to own folder .ssh
-define(USER_CONFIG, "./.ssh").
-define(TIMEOUT, 2000).

%% run test
test_getopts() ->
    start(),
    listen(),
    Result = connect(getopts),
    stop(),
    Result.

test_setopts() ->
    start(),
    listen(),
    Result = connect(setopts),
    stop(),
    Result.

%% start app

start() ->
    crypto:start(),
    ssh:start().

%% stop app

stop() ->
    ssh:stop_daemon({127, 0, 0, 1}, 7777),
    ssh:stop(),
    crypto:stop().

%% server

listen() ->
    listen(7777).

listen(Port) ->
    listen(Port, [
				  {system_dir, ?USER_CONFIG},
				  {user_dir, ?USER_CONFIG},
				  {user, "test"},
				  {password, "test"}
		 ]).

listen(Port, Opts) ->
    listen({127, 0, 0, 1}, Port, Opts).

listen(Addr, Port, Opts) ->
    case ssh:daemon(Addr, Port,   [{shell, {shell, start, []}} | Opts]) of
	{ok, SshDaemon} ->
	    io:format("SshDaemon:~p~n", [SshDaemon]);
	{error, Reason} ->
	    io:format("Error:Reason:~p~n", [Reason])
    end.

%% client

connect(RequestMsg) ->
    connect(7777, RequestMsg).

connect(Port, RequestMsg) ->
    crypto:start(),
    ssh:start(),

    Opts = [
	    {user_dir, ?USER_CONFIG},
	    {silently_accept_hosts, true},
	    {user, "test"},
	    {password, "test"}
	   ],

    case ssh:connect("127.0.0.1", Port, Opts) of
    {ok, ConnectRef} ->
	io:format("ConnectRef:~p~n", [ConnectRef]),
	session_channel(ConnectRef, RequestMsg);
    {error, Reason} ->
	io:format("Reason:~p~n", [Reason])
    end.

session_channel(ConnectRef, RequestMsg) ->
    {ok,ChannelId} = ssh_connection:session_channel(ConnectRef, infinity),
    OpenPty = ssh_connection:open_pty(ConnectRef, ChannelId, "xterm",
			    40, 40, modes_data(), 2000),
    io:format("OpenPty:~p~n", [OpenPty]),

    Shell = ssh_connection:shell(ConnectRef, ChannelId),
    io:format("Shell:~p~n", [Shell]),

    Request = case RequestMsg of
	getopts ->
	    ssh_connection:send(ConnectRef, ChannelId, <<"io:getopts().\r">>);
	setopts ->
	    ssh_connection:send(ConnectRef, ChannelId, <<"io:setopts([{encoding, latin1}]).\r">>)
    end,

    io:format("Request:~p~n", [Request]),

    receive_msg().

receive_msg() ->
    receive
	{ssh_cm,_,{data,0,0,<<"2> ">>}} = Msg ->
	    io:format("Msg:~p~n", [Msg]),
	    {ok, test_success};
	Other ->
	    io:format("Msg:~p~n", [Other]),
	    receive_msg()
    after
	2000 ->
	    {ok, test_failure}
    end.

modes_data() ->
[{tty_op_ospeed,38400},
 {tty_op_ispeed,38400},
 {vintr,3},
 {vquit,28},
 {verase,127},
 {vkill,21},
 {veof,4},
 {veol,255},
 {veol2,255},
 {vstart,17},
 {vstop,19},
 {vsusp,26},
 {vreprint,18},
 {vwerase,23},
 {vlnext,22},
 {vdiscard,15},
 {ignpar,0},
 {parmrk,0},
 {inpck,0},
 {istrip,0},
 {inlcr,0},
 {igncr,0},
 {icrnl,1},
 {iuclc,0},
 {ixon,1},
 {ixany,1},
 {ixoff,0},
 {imaxbel,1},
 {isig,1},
 {icanon,1},
 {xcase,0},
 {echo,1},
 {echoe,1},
 {echok,1},
 {echonl,0},
 {noflsh,0},
 {tostop,0},
 {iexten,1},
 {echoctl,1},
 {echoke,1},
 {pendin,0},
 {opost,1},
 {olcuc,0},
 {onlcr,1},
 {ocrnl,0},
 {onocr,0},
 {onlret,0},
 {cs7,1},
 {cs8,1},
 {parenb,0},
 {parodd,0}].
